"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var fs_1 = __importDefault(require("fs"));
var path_1 = __importDefault(require("path"));
var rimraf_1 = __importDefault(require("rimraf"));
var funcUtils_1 = require("./funcUtils");
var AzureFuncPlugin = /** @class */ (function () {
    function AzureFuncPlugin(serverless, options) {
        this.serverless = serverless;
        this.options = options;
        this.hooks = {
            "func:func": this.func.bind(this),
            "func:add:add": this.add.bind(this),
            "func:remove:remove": this.remove.bind(this)
        };
        this.commands = {
            func: {
                usage: "Add or remove functions",
                lifecycleEvents: [
                    "func",
                ],
                commands: {
                    add: {
                        usage: "Add azure function",
                        lifecycleEvents: [
                            "add",
                        ],
                        options: {
                            name: {
                                usage: "Name of function to add",
                                shortcut: "n",
                            }
                        }
                    },
                    remove: {
                        usage: "Remove azure function",
                        lifecycleEvents: [
                            "remove",
                        ],
                        options: {
                            name: {
                                usage: "Name of function to remove",
                                shortcut: "n",
                            }
                        }
                    }
                }
            }
        };
    }
    AzureFuncPlugin.prototype.func = function () {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                this.serverless.cli.log("Use the func plugin to add or remove functions within Function App");
                return [2 /*return*/];
            });
        });
    };
    AzureFuncPlugin.prototype.add = function () {
        return __awaiter(this, void 0, void 0, function () {
            var funcToAdd, exists;
            return __generator(this, function (_a) {
                if (!("name" in this.options)) {
                    this.serverless.cli.log("Need to provide a name of function to add");
                    return [2 /*return*/];
                }
                funcToAdd = this.options["name"];
                exists = fs_1.default.existsSync(funcToAdd);
                if (exists) {
                    this.serverless.cli.log("Function " + funcToAdd + " already exists");
                    return [2 /*return*/];
                }
                this.createFunctionDir(funcToAdd);
                this.addToServerlessYml(funcToAdd);
                return [2 /*return*/];
            });
        });
    };
    AzureFuncPlugin.prototype.createFunctionDir = function (name) {
        this.serverless.cli.log("Creating function dir");
        try {
            fs_1.default.mkdirSync(name);
        }
        catch (e) {
            this.serverless.cli.log("Error making directory " + e);
        }
        this.serverless.utils.writeFileSync(path_1.default.join(name, "index.js"), funcUtils_1.FuncPluginUtils.getFunctionHandler(name));
        this.serverless.utils.writeFileSync(path_1.default.join(name, "function.json"), funcUtils_1.FuncPluginUtils.getFunctionJsonString(name, this.options));
    };
    AzureFuncPlugin.prototype.addToServerlessYml = function (name) {
        this.serverless.cli.log("Adding to serverless.yml");
        var functionYml = funcUtils_1.FuncPluginUtils.getFunctionsYml(this.serverless);
        functionYml[name] = funcUtils_1.FuncPluginUtils.getFunctionSlsObject(name, this.options);
        funcUtils_1.FuncPluginUtils.updateFunctionsYml(this.serverless, functionYml);
    };
    AzureFuncPlugin.prototype.remove = function () {
        return __awaiter(this, void 0, void 0, function () {
            var funcToRemove, exists;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!("name" in this.options)) {
                            this.serverless.cli.log("Need to provide a name of function to remove");
                            return [2 /*return*/];
                        }
                        funcToRemove = this.options["name"];
                        exists = fs_1.default.existsSync(funcToRemove);
                        if (!exists) {
                            this.serverless.cli.log("Function " + funcToRemove + " does not exist");
                            return [2 /*return*/];
                        }
                        this.serverless.cli.log("Removing " + funcToRemove);
                        rimraf_1.default.sync(funcToRemove);
                        return [4 /*yield*/, this.removeFromServerlessYml(funcToRemove)];
                    case 1:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    AzureFuncPlugin.prototype.removeFromServerlessYml = function (name) {
        return __awaiter(this, void 0, void 0, function () {
            var functionYml;
            return __generator(this, function (_a) {
                functionYml = funcUtils_1.FuncPluginUtils.getFunctionsYml(this.serverless);
                delete functionYml[name];
                funcUtils_1.FuncPluginUtils.updateFunctionsYml(this.serverless, functionYml);
                return [2 /*return*/];
            });
        });
    };
    return AzureFuncPlugin;
}());
exports.AzureFuncPlugin = AzureFuncPlugin;
